

/*******************************************************************************
* File: main.c
*
* Description:
*   This code is for PSoC BB interaction with the Serial API HTML webpage.
*
* Note:
*  RS232 connects to USB port of PC
*  Mini-USB also connects to USB port of PC
*
* Hardware Dependency:
*  CY8CKIT-050 PSoC 5LP
*
********************************************************************************
* Copyright 2013, Cypress Semiconductor Corporation. All rights reserved.
* This software is owned by Cypress Semiconductor Corporation and is protected
* by and subject to worldwide patent and copyright laws and treaties.
* Therefore, you may use this software only as provided in the license agreement
* accompanying the software package from which you obtained this software.
* CYPRESS AND ITS SUPPLIERS MAKE NO WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
* WITH REGARD TO THIS SOFTWARE, INCLUDING, BUT NOT LIMITED TO, NONINFRINGEMENT,
* IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
*******************************************************************************/

#include <device.h>
#include <string.h>


#define LCD_ROWS 2u
#define LCD_COLUMNS 16u
# define TEXT "TXT "
# define LED "LED "
# define CLEAR "CLR "
# define n 4
# define resetLCDPosition row = 0u; pos = 0u;


uint8 pos = 0u;
uint8 row = 0u;
char8 ch;                                           /* Data received from the Serial port */
    

int parse(char* buffer) {                           /* Parse returns 2 for Text & 1 for LED (0 otherwise) */
      if (strcmp(buffer, TEXT)==0) {
        LCD_WriteControl(LCD_CLEAR_DISPLAY);        // clear LCD
        resetLCDPosition                            // reset LCD cursor
            
        return 2;                                   // change mode to text
        
    } else if (strcmp(buffer, LED) == 0) {
        resetLCDPosition                            // reset LCD cursor
        
        return 1;                                   // change mode to PWM for LED
        
    } else if (strcmp(buffer, CLEAR) == 0) {
        LCD_WriteControl(LCD_CLEAR_DISPLAY);        // clear LCD
        UART_PutString(" *Display cleared* \n\r");  // transmit feedback to user on HTML
        resetLCDPosition                            // reset LCD cursor
        ch = UART_GetChar();                        // Get new byte (ASCII --> uint8)
        return 0;
        
    } else{
        return 0;
    }
}

void shiftBuffer (char* buffer) {                   /* Shift and add to n-char buffer */
    int i;
    for(i=0;i<n-1;i++) {
        buffer[i]=buffer[i+1];
    }
    buffer[n-1]=ch;                                 // Last element in buffer is most recent data
}

void shiftLCDPos() {                                /* Change row and pos for next char */
                      
    pos++;
    
    if(pos % 16 == 0u) {                            // If LCD cursor reaches right edge of display (count=16)
        pos = 0u;                                   // reset to left side
        row++;                                      // next row
        row%=2;                                     // Alternate rows to use both rows of 16 to display message
    }
    
}

int isPrintable (int dec) {                         /* Check that it is a valid printable ASCII char */
    if (dec > 31 && dec < 127) {
        return 1;
    } else {
        return 0;
    }
    
}


/*******************************************************************************
* Function Name: main
********************************************************************************
*
* Summary:
* Main function performs following functions:
*
* 1: Start the LCD, UART, and PWM
*
* 2: On reset, transmit "*Device Reset*" and turn LED off
*
* 3: Upon receiving data, variable 'decode' is 1 (text) or 0 (LED):
*
*       a. "TXT " --> indicates that further received data is text; print to LCD
*
*       b. "LED " --> indicates that further received data is for LED; set PWM compare value
*
*       c. "CLR " --> indicates that the "Clear LCD" button was pressed; clear the LCD
*
* Parameters:
*  None.
*
* Return:
*  None.
*
*******************************************************************************/
int main()
{
    int decode = 1;         // 1 = TEXT, 0 = LED  (Default = text)
    char buff[n];           // n char buffer 
    
    CyGlobalIntEnable;
    
    /* Start and initialize the LCD, UART, and PWM */
    LCD_Start();
        
    UART_Start();    
      
    PWM_Start();
    
    /* Send reset string and turn off LED */
    
    UART_PutString(" *Device reset* \n\r");
    
    PWM_WriteCompare(0);
    
    for(;;)
    {
        if (UART_ReadRxStatus() == UART_RX_STS_FIFO_NOTEMPTY) {     // If receiving data
                   
            /* Get most recent received byte (ASCII --> uint8) */
            ch = UART_GetChar();
            
            /* Shift and Check n-char buffer */
            shiftBuffer(buff);
            
            int code = parse (buff);      
            /* parsing TEXT returns 2, parsing LED returns 1, else return 0 */
            
            if (code > 0) {                     // if buffer is a special indicator
                
                decode = code-1;                // set decode to 1 or 0 
                
                ch = UART_GetChar();            // Get new byte (ASCII --> uint8)
                
            } else {
                
                /* Handle Text */
                if (decode == 1 && isPrintable(ch)) {
                    
                    shiftLCDPos();              /* Shift LCD Position */
                    
                    LCD_PutChar(ch);            /* Print the received character on LCD */
                        
                    UART_PutChar(ch);           /* Give feedback to HTML Page */
                    
                    LCD_Position(row, pos);     /* Move LCD Cursor */
                }
                /* Handle PWM */
                // unaffected by ASCII values in "TXT " (in decimal, T=84, X=88, [SPACE]=32)
                else if (decode == 0 && ch!=84 && ch!=88 && ch!=32) { 
                    
                    PWM_WriteCompare(ch);       // set Compare Value 
                    
                }
            }
        }
    }
}


/* [] END OF FILE */
