/* Define any variables used later */


/* Serial Communication Variables */
// the selected peripheral device
var port = null;
// reader object
var reader = null;
// writer object
var writer = null;

// Sending ledUpdate tells the PSoC to interpret as LED brightness
const ledUpdate = "LED ";
// Sending txtUpdate tells the PSoC to interpret as text
const txtUpdate = "TXT ";
// Sending clrUpdate tells the PSoC to clear the LCD
const clrUpdate = "CLR ";


/* HTML Elements  */
// status connection (text is updated with connection status)
const stat = document.querySelector("#status"); // HTML element w/ ID of "status"
// Text Input Box
 input = document.getElementById("data"); // HTML element w/ ID of "data"
// Slider for LED
const slider = document.getElementById("bar"); // HTML element w/ ID of "status"
// Slider value text
const LEDvalue = document.getElementById("pwm"); // HTML element w/ ID of "status"
// Device Log ("Terminal")
const text = document.getElementById("main_text"); // HTML element w/ ID of "main_text"
// Send Data button
const send = document.querySelector("#send"); // HTML element w/ ID of "send"
// Disconnect button
const disconn = document.querySelector("#disconnect"); // HTML element w/ ID of "disconnect"
// Clear Log button
const clearlog = document.getElementById("clearLog"); // HTML element w/ ID of "clearLog"
/* Filter on devices (i.e. Arduino Uno USB Vendor/Product IDs -- provided in Google's documentation)
//const filters = [{ usbVendorId: ####, usbProductId: #### }];
// Ex. usbVendorId: 1659, usbProductId: 9123*/



/* Interacting with HTML: Event Listeners for text and slider function */

/* SENDING TEXT */
send.addEventListener("click", async () => {
 // when 'Send Data' button is clicked
 // write typed string to serial port and console & reset text box
 await textTx(input.value);
 console.log(input.value);
 input.value = "";

 // new line in Device Log after text
 setTimeout(() => {
   text.innerHTML += "<br />";
 }, 500);
});

input.addEventListener("keydown", async function(event) {
  // Pressing "Enter" on the text input box triggers 'Send Data'
  if (event.key === "Enter") {
	send.click();
  }
});


/* SENDING LED VALUES */

// Upon clicking on LED slider
slider.addEventListener("mousedown", async (e) => {
 // Tell PSoC to interpret as LED value
 await textTx(ledUpdate);
});

// On any change in slider value, the value is sent to PSoC
slider.oninput = async function () {

 //purely update the html text
 LEDvalue.innerHTML = "Value: " + this.value + "/255";

 // write the slider value to serial port
 const uint8 = new Uint8Array(1);
 uint8[0] = this.value;
 await ledTx(uint8);
};

// Upon releasing LED slider
slider.addEventListener("mouseup", async (e) => {

 // Return to default text state after releasing slider
 await textTx(txtUpdate);
});


/* Clear buttons */
// Clear LCD Button
clear.addEventListener("click", async (e) => {
 await textTx(clrUpdate);
 console.log("LCD cleared");
});
// Clear Device Log text
clearlog.addEventListener("click", async (e) => {
 text.innerHTML = "";
});


/* Disconnecting */
// User clicked the 'disconnect' button to close the serial port.
disconn.addEventListener("click", async () => {
 console.log("disconnecting...");
 disconnect();
 stat.innerHTML = "STATUS: Disconnected"; // update status on screen
 console.log("Port disconnected!"); // update status in console
});





/* [FUNCTIONS] */

/* CONNECT() */
async function connect() {
 try {
   // Prompt user to select any serial port. [option to add filters - see const filters above]
   // ADD FILTERS: port = await navigator.serial.requestPort({ filters });
   port = await navigator.serial.requestPort();
 } catch (err) {
   console.log("Error on request: ", err);
 }

 try {
   //set baud rate to user's input. [option to prompt user for custom baud rate or specific bps]
   //CUSTOM BAUD RATE: await port.open({ baudRate: prompt("Baud rate") });
   //SPECIFIC BAUD RATE: await port.open({ baudRate: #### });
   await port.open({ baudRate: 9600 });
   const { usbVendorId, usbProductId } = port.getInfo();
 } catch (err) {
   console.log("Error on open: ", err);
 }
 console.log("Port Connected!", port); // update status in console
 stat.innerHTML = "STATUS: Connected"; // update status on HTML/screen

 textRx(); // constantly receive information on loop until disconnected


 if (port.writable) {
   writer = port.writable.getWriter();
   await writer.ready; // resolve any potential stream backpressure
 }
}


/* DISCONNECT() */
async function disconnect() {
 // Cancel reader, release writer, and close port
 if (reader) {
   await reader.cancel();
   reader = null;
 }
 if (writer) {
   await writer.releaseLock();
   writer = null;
 }
 if (port) {
   await port.close();
   port = null;
 }
}


/* TEXTTX() */
async function textTx(msg) {
 // msg is a string
 const encoder = new TextEncoder(); // encodes string as uint8's
 if (writer) {
   const encMessage = encoder.encode(msg + "\n\r"); // encode message
   // resolve any potential stream backpressure
   await writer.ready.catch((err) => {
     console.error(`Ready error: ${err}`);
   });
   //transmit encoded message
   await writer.write(encMessage).catch((err) => {
     console.error(`Chunk error: ${err}`);
   });
   // resolve any potential stream backpressure
   await writer.ready;
 }
}


/* LEDTX() */
async function ledTx(val) {
 // val is a Uint8 Array
 if (writer) {
   // resolve any potential stream backpressure
   await writer.ready.catch((err) => {
     console.error(`Ready error: ${err}`);
   });
   //transmit uint8 array with LED value
   await writer.write(val).catch((err) => {
     console.error(`Chunk error: ${err}`);
   });
   // resolve any potential stream backpressure
   await writer.ready;
 }
}


/* TEXTRX() */
async function textRx() {
 console.log("Read Loop Init");
 // resolve errors related to port = null
 if (!port) {
   return;
 }
 const decoder = new TextDecoder(); // decodes uint8's to a string

 //continuously read (loop)
 if (port.readable) {
   reader = port.readable.getReader();
   console.log("Read Loop Started");
   while (true) {
     const { value, done } = await reader.read();

	/* Handle Rx Data */
     if (text.innerHTML.substr(-2) === "t*") { 
       // Reset slider and new line (Device Log) after *Device Reset*
       text.innerHTML += "<br />";
       slider.value = 0;
       LEDvalue.innerHTML = "Value: " + "0" + "/255";

     } else if (text.innerHTML.substr(-2) === "d*" || text.innerHTML.substr(-3) === "LED") { 
       //New line (Device Log) after *Display Cleared* or LED
       text.innerHTML += "<br />";
     }

     text.innerHTML += decoder.decode(value); // add received chars to the Device Log
     text.scrollTop = text.scrollHeight; // Auto scroll to bottom of Device L9og


     // done = true if reader is canceled; needs to release
     if (done) {
       reader.releaseLock();
       break;
     }
   }
 }
 console.log("Read Loop Stopped. Closing Serial Port.");
}

/* Event Listeners for serial port -- more info on API Docs */
// Automatically inform user a port has been detected. (ex. a USB is plugged in)
navigator.serial.addEventListener("connect", (event) => {
 alert("A port is available!");
 console.log(event);
});
// Automatically inform user a port has been disconnected. (ex. a USB is unplugged)
navigator.serial.addEventListener("disconnect", (event) => {
 alert("A port was disconnected!");
 console.log(event);
});


// Prints browser-API compatability to console
if ("serial" in navigator) {
 console.log("// The Web Serial API is supported");
} else {
 console.log("// The Web Serial API is not supported");
}
if ("serial" in navigator && "forget" in SerialPort.prototype) {
 console.log("// forget() is supported.");
} else {
 console.log("// forget() is not supported.");
}